@@ -15,6 +15,7 @@
 
 from oslo_log import log
 from oslo_utils import importutils
+import retrying
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -28,6 +29,7 @@ from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_utils
 from ironic.drivers.modules.redfish import utils as redfish_utils
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
@@ -67,6 +69,7 @@ RESCUE_PROPERTIES = {
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
+COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
 
@@ -141,19 +144,31 @@ def _parse_instance_info(node):
     return deploy_info
 
 
-def _insert_vmedia(task, boot_url, boot_device):
+def _test_retry(exception):
+    if isinstance(exception, sushy.exceptions.ServerSideError):
+        # On some Dell hw, the eject media may still be in progress
+        # https://storyboard.openstack.org/#!/story/2008504
+        LOG.warning("Boot media insert failed for node %(node)s, "
+                    "will retry after 3 seconds",
+                    {'node': exception.node_uuid})
+        return True
+    return False
+
+
+@retrying.retry(wait_fixed=3000, stop_max_attempt_number=3,
+                retry_on_exception=_test_retry)
+def _insert_vmedia(task, managers, boot_url, boot_device):
     """Insert bootable ISO image into virtual CD or DVD
 
     :param task: A task from TaskManager.
+    :param managers: A list of System managers.
     :param boot_url: URL to a bootable ISO image
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device not in v_media.media_types:
                 continue
@@ -169,8 +184,12 @@ def _insert_vmedia(task, boot_url, boot_device):
 
                 continue
 
-            v_media.insert_media(boot_url, inserted=True,
-                                 write_protected=True)
+            try:
+                v_media.insert_media(boot_url, inserted=True,
+                                     write_protected=True)
+            except sushy.exceptions.ServerSideError as e:
+                e.node_uuid = task.node.uuid
+                raise
 
             LOG.info("Inserted boot media %(boot_url)s into "
                      "%(boot_device)s for node "
@@ -183,19 +202,18 @@ def _insert_vmedia(task, boot_url, boot_device):
         _('No suitable virtual media device found'))
 
 
-def eject_vmedia(task, boot_device=None):
+def _eject_vmedia(task, managers, boot_device=None):
     """Eject virtual CDs and DVDs
 
     :param task: A task from TaskManager.
+    :param managers: A list of System managers.
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
         eject everything (default).
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device and boot_device not in v_media.media_types:
                 continue
@@ -212,19 +230,32 @@ def eject_vmedia(task, boot_device=None):
                           'boot_device': v_media.name})
 
 
-def _has_vmedia_device(task, boot_device):
-    """Indicate if device exists at any of the managers
+def eject_vmedia(task, boot_device=None):
+    """Eject virtual CDs and DVDs
 
     :param task: A task from TaskManager.
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
-        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`.
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
+        eject everything (default).
+    :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+        found on the node.
     """
     system = redfish_utils.get_system(task.node)
+    _eject_vmedia(task, system.managers, boot_device=boot_device)
 
-    for manager in system.managers:
+
+def _has_vmedia_device(managers, boot_device):
+    """Indicate if device exists at any of the managers
+
+    :param managers: A list of System managers.
+    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`.
+    """
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device in v_media.media_types:
                 return True
+    return False
 
 
 def _parse_deploy_info(node):
@@ -279,7 +310,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
     `[instance_info]image_source` node property.
     """
 
-    capabilities = ['iscsi_volume_boot', 'ramdisk_boot']
+    capabilities = ['iscsi_volume_boot', 'ramdisk_boot',
+                    'ramdisk_boot_configdrive']
 
     def __init__(self):
         """Initialize the Redfish virtual media boot interface.
@@ -342,6 +374,19 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
         deploy_utils.validate_image_properties(task.context, d_info, props)
 
+    def _validate_vendor(self, task):
+        vendor = task.node.properties.get('vendor')
+        if not vendor:
+            return
+
+        if 'Dell' in vendor.split():
+            raise exception.InvalidParameterValue(
+                _("The %(iface)s boot interface is not suitable for node "
+                  "%(node)s with vendor %(vendor)s, use "
+                  "idrac-redfish-virtual-media instead")
+                % {'iface': task.node.boot_interface,
+                   'node': task.node.uuid, 'vendor': vendor})
+
     def validate(self, task):
         """Validate the deployment information for the task's node.
 
@@ -353,6 +398,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :raises: InvalidParameterValue on malformed parameter(s)
         :raises: MissingParameterValue on missing parameter(s)
         """
+        self._validate_vendor(task)
         self._validate_driver_info(task)
 
         if task.driver.storage.should_write_image(task):
@@ -420,9 +466,11 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         if CONF.debug and 'ipa-debug' not in ramdisk_params:
             ramdisk_params['ipa-debug'] = '1'
 
+        managers = redfish_utils.get_system(task.node).managers
+
         if config_via_floppy:
 
-            if _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY):
+            if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_FLOPPY):
                 # NOTE (etingof): IPA will read the diskette only if
                 # we tell it to
                 ramdisk_params['boot_method'] = 'vmedia'
@@ -430,9 +478,9 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                 floppy_ref = image_utils.prepare_floppy_image(
                     task, params=ramdisk_params)
 
-                eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
+                _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
                 _insert_vmedia(
-                    task, floppy_ref, sushy.VIRTUAL_MEDIA_FLOPPY)
+                    task, managers, floppy_ref, sushy.VIRTUAL_MEDIA_FLOPPY)
 
                 LOG.debug('Inserted virtual floppy with configuration for '
                           'node %(node)s', {'node': task.node.uuid})
@@ -447,8 +495,10 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         iso_ref = image_utils.prepare_deploy_iso(task, ramdisk_params,
                                                  mode, d_info)
 
-        eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        del managers
 
         boot_mode_utils.sync_boot_mode(task)
 
@@ -474,12 +524,14 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         LOG.debug("Cleaning up deploy boot for "
                   "%(node)s", {'node': task.node.uuid})
 
-        eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
+        managers = redfish_utils.get_system(task.node).managers
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
         image_utils.cleanup_iso_image(task)
 
         if (config_via_floppy
-                and _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY)):
-            eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
+                and _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_FLOPPY)):
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
 
             image_utils.cleanup_floppy_image(task)
 
@@ -505,8 +557,12 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
 
+        self._eject_all(task)
+
+        boot_mode_utils.sync_boot_mode(task)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
+
         boot_option = deploy_utils.get_boot_option(node)
-        self.clean_up_instance(task)
         iwdi = node.driver_internal_info.get('is_whole_disk_image')
         if boot_option == "local" or iwdi:
             self._set_boot_device(task, boot_devices.DISK, persistent=True)
@@ -531,12 +587,26 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
             params.update(root_uuid=root_uuid)
 
-        deploy_info = _parse_deploy_info(node)
-        iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
-        eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        managers = redfish_utils.get_system(task.node).managers
 
-        boot_mode_utils.sync_boot_mode(task)
+        deploy_info = _parse_deploy_info(node)
+        configdrive = node.instance_info.get('configdrive')
+        iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        if configdrive and boot_option == 'ramdisk':
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            cd_ref = image_utils.prepare_configdrive_image(task, configdrive)
+            try:
+                _insert_vmedia(task, managers, cd_ref,
+                               sushy.VIRTUAL_MEDIA_USBSTICK)
+            except exception.InvalidParameterValue:
+                raise exception.InstanceDeployFailure(
+                    _('Cannot attach configdrive for node %s: no suitable '
+                      'virtual USB slot has been found') % node.uuid)
+
+        del managers
 
         self._set_boot_device(task, boot_devices.CDROM, persistent=True)
 
@@ -544,6 +614,23 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                   "%(device)s", {'node': task.node.uuid,
                                  'device': boot_devices.CDROM})
 
+    def _eject_all(self, task):
+        managers = redfish_utils.get_system(task.node).managers
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        d_info = task.node.driver_info
+        config_via_floppy = d_info.get('config_via_floppy')
+        if config_via_floppy:
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
+
+        boot_option = deploy_utils.get_boot_option(task.node)
+        if (boot_option == 'ramdisk'
+                and task.node.instance_info.get('configdrive')):
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            image_utils.cleanup_disk_image(task, prefix='configdrive')
+
+        image_utils.cleanup_iso_image(task)
+
     def clean_up_instance(self, task):
         """Cleans up the boot of instance.
 
@@ -555,14 +642,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         LOG.debug("Cleaning up instance boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        d_info = task.node.driver_info
-        config_via_floppy = d_info.get('config_via_floppy')
-        if config_via_floppy:
-            eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-        image_utils.cleanup_iso_image(task)
+        self._eject_all(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
     @classmethod
     def _set_boot_device(cls, task, device, persistent=False):
